package com.avinash.gotluru.ingenico_device;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.pm.PackageManager;
import android.os.Build;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;

import com.ingenico.mpos.sdk.Ingenico;
import com.ingenico.mpos.sdk.callbacks.ApplicationSelectionCallback;
import com.ingenico.mpos.sdk.callbacks.DeviceSetupCallback;
import com.ingenico.mpos.sdk.callbacks.LoginCallback;
import com.ingenico.mpos.sdk.callbacks.TransactionCallback;
import com.ingenico.mpos.sdk.constants.ResponseCode;
import com.ingenico.mpos.sdk.data.Amount;
import com.ingenico.mpos.sdk.data.UserProfile;
import com.ingenico.mpos.sdk.request.CreditSaleTransactionRequest;
import com.ingenico.mpos.sdk.response.TransactionResponse;
import com.ingenico.mpos.sdk.views.LedPairingView;
import com.roam.roamreaderunifiedapi.callback.LedPairingCallback;
import com.roam.roamreaderunifiedapi.callback.LedPairingConfirmationCallback;
import com.roam.roamreaderunifiedapi.callback.SearchListener;
import com.roam.roamreaderunifiedapi.constants.CommunicationType;
import com.roam.roamreaderunifiedapi.constants.DeviceType;
import com.roam.roamreaderunifiedapi.data.ApplicationIdentifier;
import com.roam.roamreaderunifiedapi.data.Device;
import com.roam.roamreaderunifiedapi.data.LedSequence;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import io.flutter.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * IngenicoDevicePlugin
 */
public class IngenicoDevicePlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;
    public Activity activity;
    String apikey, baseUrl, clientVersion, username, password;
    public TransactionResponse TResponse ;




    /**
     * @param deviceList
     * <p>
     * Which gives the device list
     */
    ArrayList<Device> deviceList;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "ingenico_device");
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        }
        else if (call.method.equals("Initialize")) {
            apikey = call.argument("apiKey");
            baseUrl = call.argument("baseUrl");
            clientVersion = call.argument("clientVersion");
            Initialize(apikey, baseUrl, clientVersion);
            result.success("Initialize Done");
        }
        else if (call.method.equals("Login")) {
            username = call.argument("username");
            password = call.argument("password");
            Login(username, password);
            result.success("Login Done");
        }
        else if (call.method.equals("Search")) {
            List<Device> devices = BluetoothSearch();
            result.success(devices);
        }
        else if (call.method.equals("SetDeviceType")) {
            setDeviceType();
            result.success("true");
        }
        else if (call.method.equals("SetUpDevice")) {
            setUpDevice();
            result.success("true");
        }
        else if (call.method.equals("CreditSale")) {
            TransactionResponse response = processCreditTransaction("USD",1000);
            result.success(response);
        } else {
            result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        this.activity = binding.getActivity();

    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        this.activity = null;
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        this.activity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivity() {
        this.activity = null;
    }

    /**
     * Ingenico Implementation
     * */

    /**
     * Initialize
     *
     * @param apikey
     * @param baseUrl
     * @param clientVersion
     */

    private void Initialize(String apikey, String baseUrl, String clientVersion) {
        Ingenico.getInstance().setLogging(true);
        Ingenico.getInstance().initialize(this.activity, baseUrl, apikey, clientVersion);
    }

    /**
     * Initialize
     *
     * @param username username for login the the Ingenico Gateway
     * @param password Password for login the the Ingenico Gateway
     */
    private void Login(String username, String password) {
        Ingenico.getInstance().user().login(username, password, new LoginCallback() {
            @Override
            public void done(Integer integer, UserProfile userProfile) {
                if (ResponseCode.Success == integer) {
                    Log.d("Avinash", "done: Login" + userProfile);
                } else {
                    Log.d("Avinash", "done: Login Failed");
                }
            }
        });
    }

    /**
     * BluetoothSearch
     */
    private ArrayList<Device> BluetoothSearch() {
        deviceList = new ArrayList<>();
        checkPermissions();
        Ingenico.getInstance().c.setDeviceType(DeviceType.MOBY5500);
        Ingenico.getInstance().device().search(activity, true, 15000L, new SearchListener() {
            @Override
            public void onDeviceDiscovered(Device device) {
                if(device.getDeviceType() == DeviceType.MOBY5500 && device.getConnectionType() == CommunicationType.Bluetooth){
                    deviceList.add(device);
                    connectDevice(deviceList.get(0));
                }
            }

            @Override
            public void onDiscoveryComplete() {
            }
        });
        return deviceList;
    }

    /**
     * Permission Check
     */
    private void checkPermissions() {
        if (Build.VERSION.SDK_INT > Build.VERSION_CODES.LOLLIPOP_MR1) {
            if (ActivityCompat.checkSelfPermission(activity, Manifest.permission.BLUETOOTH) != PackageManager.PERMISSION_GRANTED
                    || ActivityCompat.checkSelfPermission(activity, Manifest.permission.BLUETOOTH_ADMIN) != PackageManager.PERMISSION_GRANTED
                    || ActivityCompat.checkSelfPermission(activity, Manifest.permission.MODIFY_AUDIO_SETTINGS) != PackageManager.PERMISSION_GRANTED
                    || ActivityCompat.checkSelfPermission(activity, Manifest.permission.RECORD_AUDIO) != PackageManager.PERMISSION_GRANTED
                    || ActivityCompat.checkSelfPermission(activity, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED
                    || ActivityCompat.checkSelfPermission(activity, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                ActivityCompat.requestPermissions(activity, new String[]{
                        Manifest.permission.BLUETOOTH,
                        Manifest.permission.BLUETOOTH_ADMIN,
                        Manifest.permission.ACCESS_FINE_LOCATION,
                        Manifest.permission.ACCESS_COARSE_LOCATION,
                        Manifest.permission.MODIFY_AUDIO_SETTINGS,
                        Manifest.permission.RECORD_AUDIO,
                }, 10);
            }
        }
    }

    /**
     * Set Device Type
     *
     * example MOB55,MOBY8500.. Please refer Device Type
     */
    private void setDeviceType() {
        Ingenico.getInstance().c.setDeviceType(DeviceType.MOBY5500);
    }

    /**
     * Setup Device
     */
    private void setUpDevice() {
        Ingenico.getInstance().c.setup(new DeviceSetupCallback() {
            @Override
            public void done(Integer integer) {
                if (ResponseCode.Success == integer) {
                    Toast.makeText(activity, "Setup Complete", Toast.LENGTH_LONG).show();
                }
            }
        });
    }

    private TransactionResponse processCreditTransaction(String currency, int total) {
        final AlertDialog.Builder alertDialog = new AlertDialog.Builder(activity);
        CreditSaleTransactionRequest creditSaleTransactionRequest = new CreditSaleTransactionRequest(new Amount(currency, total), false);
        Ingenico.getInstance().payment().processCreditSaleTransactionWithCardReader(
                creditSaleTransactionRequest, new TransactionCallback() {
                    @Override
                    public void updateProgress(Integer integer, String s) {

                    }

                    @Override
                    public void applicationSelection(List<ApplicationIdentifier> list, ApplicationSelectionCallback applicationSelectionCallback) {

                    }

                    @Override
                    public void done(Integer integer, TransactionResponse transactionResponse) {
                        if (ResponseCode.Success == integer) {
                            TResponse = transactionResponse;
                            alertDialog.setTitle("Transaction Response").setMessage(transactionResponse.toString()).setPositiveButton("Okay", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    dialog.dismiss();
                                }
                            });
                            alertDialog.create().show();
                        }
                    }
                }
        );
        return TResponse;
    }

    private void connectDevice(final Device device){
        Log.d("Avinash", "connectToReader::" + device);
        if (null != device) {
            Ingenico.getInstance().device().select(device);
            if (device.getDeviceType() == DeviceType.MOBY5500) {
                connect(activity);
            }
        }
    }
    public static void connect(final Context context) {
        // Step 1: Initialize device
        Ingenico.getInstance().device().initialize(context, new LedPairingCallback() {
            @Override
            public void confirmLedSequence(List<LedSequence> sequenceList, LedPairingConfirmationCallback ledPairingConfirmationCallback) {
                Log.v("TAG", "LedPairingCallback::confirmLedSequence");
                showPairingDialog(context,sequenceList,ledPairingConfirmationCallback);
            }

            @Override
            public void notSupported() {
                Log.v("TAG", "LedPairingCallback::notSupported");
            }

            @Override
            public void success() {
                Log.v("TAG", "LedPairingCallback::success");
            }

            @Override
            public void fail() {
                Log.v("TAG", "LedPairingCallback::fail");
            }

            @Override
            public void canceled() {
                Log.v("TAG", "LedPairingCallback::canceled");
            }
        });
    }

    private static AlertDialog showPairingDialog(Context context,
                                                 List<LedSequence> sequenceList,
                                                 final LedPairingConfirmationCallback ledPairingConfirmationCallback) {
        AlertDialog.Builder builder;
        builder = new AlertDialog.Builder(context);
        builder.setTitle("Confirm Led Sequence");
        View dialogView;
        dialogView = LayoutInflater.from(context).inflate(R.layout.dialog_pairing_led, null);
        LedPairingView ledPairingView = dialogView.findViewById(R.id.pairingLedView);
        builder.setView(dialogView);
        builder.setCancelable(false);
        builder.setNeutralButton("Restart",
                new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        new Thread(new Runnable() {
                            @Override
                            public void run() {
                                ledPairingConfirmationCallback.restartLedPairingSequence();
                            }
                        }).start();
                    }
                });
        builder.setPositiveButton("Confirm",
                new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        new Thread(new Runnable() {
                            @Override
                            public void run() {
                                ledPairingConfirmationCallback.confirm();
                            }
                        }).start();
                    }
                });
        builder.setNegativeButton("Cancel",
                new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        new Thread(new Runnable() {
                            @Override
                            public void run() {
                                ledPairingConfirmationCallback.cancel();
                            }
                        }).start();
                    }
                });
        AlertDialog dialog = builder.create();
        dialog.show();
        ledPairingView.show(sequenceList);
        return dialog;
    }
}

