import 'dart:async';
import 'package:flutter/services.dart';
export 'device.dart';

class IngenicoDevice {
  /// [_channel] to communicate wiht the native platforms
  static const MethodChannel _channel = const MethodChannel('ingenico_device');

  /// Ingenico [ingenicoInitialize]
  /// [apikey] --> API_KEY of Ingenico SDK
  /// [baseUrl] --> BASE_URL
  /// [clientVersion] --> CLIENT_VERSION 4.3.2
  static Future<void> ingenicoInitialize(
    String? apiKey,
    String? baseUrl,
    String? clientVersion,
  ) async {
    await _channel.invokeMethod('Initialize', {
      'apiKey': apiKey,
      'baseUrl': baseUrl,
      'clientVersion': clientVersion,
    });
  }

  /// Ingenico [ingenicoLogin]
  /// [username] --> Username for Gateway login
  /// [password] --> Password for Gateway login
  static Future<void> ingenicoLogin(
    String? username,
    String? password,
  ) async {
    await _channel.invokeMethod('Login', {
      'username': username,
      'password': password,
    });
  }

  /// Ingenico [search]
  /// Search for ingenico payment devices
  static Future<dynamic> search() async {
    await _channel.invokeMethod('Search');
  }

  /// Ingenico [setDeviceType]
  /// Set type of ingenico payment device
  static Future<dynamic> setDeviceType() async {
    await _channel.invokeMethod('SetDeviceType');
  }

  /// Ingenico [setUpDevice]
  /// Setup the device for transaction
  static Future<dynamic> setUpDevice() async {
    await _channel.invokeMethod('SetUpDevice');
  }

  /// Ingenico [creditSale]
  /// [currency] --> Country Currency Code example "INR", "USD"....etc
  /// [total] --> Total amount for the transaction
  static Future<dynamic> creditSale(String currency, int total) async {
    await _channel
        .invokeMethod('CreditSale', {'Currency': currency, 'Total': total});
  }
}
